--------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
--------------------------------------------------------------------------------
--
-- *****************************************************************************
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"           **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND         **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,          **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,          **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION             **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,       **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE        **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY                **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE                 **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR          **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF         **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS         **
-- ** FOR A PARTICULAR PURPOSE.                                               **
-- **                                                                         **
-- ** (c) Copyright 2004 Xilinx Inc.                                          **
-- ** All rights reserved.                                                    **
-- *****************************************************************************
--
--------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.00.a
-- Description:       User logic module.
-- Date:              Mon Mar 01 08:48:45 2004 (by Create/Import Peripheral Wizard)
-- VHDL-Standard:     VHDL'93
--------------------------------------------------------------------------------
-- Naming Conventions:
-- 	active low signals:                    "*_n"
-- 	clock signals:                         "clk", "clk_div#", "clk_#x"
-- 	reset signals:                         "rst", "rst_n"
-- 	generics:                              "C_*"
-- 	user defined types:                    "*_TYPE"
-- 	state machine next state:              "*_ns"
-- 	state machine current state:           "*_cs"
-- 	combinatorial signals:                 "*_com"
-- 	pipelined or register delay signals:   "*_d#"
-- 	counter signals:                       "*cnt*"
-- 	clock enable signals:                  "*_ce"
-- 	internal version of output port:       "*_i"
-- 	device pins:                           "*_pin"
-- 	ports:                                 "- Names begin with Uppercase"
-- 	processes:                             "*_PROCESS"
-- 	component instantiations:              "<ENTITY_>I_<#|FUNC>"
--------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v1_00_b;
use proc_common_v1_00_b.proc_common_pkg.all;

-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

--------------------------------------------------------------------------------
-- Definition of Generics:
--   C_DWIDTH                     -- User logic data bus width
--   C_NUM_CE                     -- User logic chip enable bus width
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Data                  -- Bus to IP data bus
--   Bus2IP_BE                    -- Bus to IP byte enables
--   Bus2IP_RdCE                  -- Bus to IP read chip enable for user logic
--   Bus2IP_WrCE                  -- Bus to IP write chip enable user logic
--   IP2Bus_Data                  -- IP to Bus data bus for user logic
--   IP2Bus_Ack                   -- IP to Bus acknowledgement
--   IP2Bus_Retry                 -- IP to Bus retry response
--   IP2Bus_Error                 -- IP to Bus error response
--   IP2Bus_ToutSup               -- IP to Bus timeout suppress
--
--------------------------------------------------------------------------------
-- Entity section
--------------------------------------------------------------------------------

entity user_logic is
	generic
	(
		-- ADD USER GENERICS BELOW THIS LINE ---------------
		--USER generics added here
		-- ADD USER GENERICS ABOVE THIS LINE ---------------

		-- DO NOT EDIT BELOW THIS LINE ---------------------
		-- Bus protocol parameters, do not add to or delete
		C_DWIDTH	: integer	:= 8;
		C_NUM_CE	: integer	:= 4
		-- DO NOT EDIT ABOVE THIS LINE ---------------------
	);
	port
	(
		-- ADD USER PORTS BELOW THIS LINE ------------------
		digit           : out std_logic_vector(0 to 3);
		segment         : out std_logic_vector(0 to 7);
		-- ADD USER PORTS ABOVE THIS LINE ------------------

		-- DO NOT EDIT BELOW THIS LINE ---------------------
		-- Bus protocol ports, do not add to or delete
		Bus2IP_Clk	    : in	std_logic;
		Bus2IP_Reset	: in	std_logic;
		Bus2IP_Data	    : in	std_logic_vector(0 to C_DWIDTH-1);
		Bus2IP_BE	    : in	std_logic_vector(0 to C_DWIDTH/8-1);
		Bus2IP_RdCE	    : in	std_logic_vector(0 to C_NUM_CE-1);
		Bus2IP_WrCE	    : in	std_logic_vector(0 to C_NUM_CE-1);
		IP2Bus_Data	    : out	std_logic_vector(0 to C_DWIDTH-1);
		IP2Bus_Ack	    : out	std_logic;
		IP2Bus_Retry	: out	std_logic;
		IP2Bus_Error	: out	std_logic;
		IP2Bus_ToutSup	: out	std_logic
		-- DO NOT EDIT ABOVE THIS LINE ---------------------
	);
end entity user_logic;

--------------------------------------------------------------------------------
-- Architecture section
--------------------------------------------------------------------------------

architecture IMP of user_logic is

	----------------------------------------
	-- User logic S/W accessible registers
	----------------------------------------
	signal reg0	            : std_logic_vector(0 to C_DWIDTH-1);
	signal reg1	            : std_logic_vector(0 to C_DWIDTH-1);
	signal reg2	            : std_logic_vector(0 to C_DWIDTH-1);
	signal reg3	            : std_logic_vector(0 to C_DWIDTH-1);
	signal reg_write_select	: std_logic_vector(0 to 3);
	signal reg_read_select	: std_logic_vector(0 to 3);
	signal digit_select	    : std_logic_vector(0 to 1);
	signal digit_i          : std_logic_vector(0 to 3);
	signal segment_i        : std_logic_vector(0 to 7);
    signal mhertz_count	    : std_logic_vector(5 downto 0);
    signal khertz_count	    : std_logic_vector(9 downto 0);
    signal mhertz_en	    : std_logic;
    signal khertz_en   	    : std_logic;

begin

	----------------------------------------
	-- Map user logic S/W register read/write select signal
	----------------------------------------
	reg_write_select <= Bus2IP_WrCE;
	reg_read_select <= Bus2IP_RdCE;

	----------------------------------------
	-- User logic S/W accessible registers write example
	----------------------------------------
	REG_WRITE_PROC : process( Bus2IP_Clk ) is
	begin
		if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
			if Bus2IP_Reset = '1' then
				reg0 <= (others => '0');
				reg1 <= (others => '0');
				reg2 <= (others => '0');
				reg3 <= (others => '0');
			else
				case reg_write_select is
					when "1000" => reg0 <= Bus2IP_Data;
					when "0100" => reg1 <= Bus2IP_Data;
					when "0010" => reg2 <= Bus2IP_Data;
					when "0001" => reg3 <= Bus2IP_Data;
					when others => null;
				end case;
			end if;
		end if;
	end process REG_WRITE_PROC;

	----------------------------------------
	-- User logic S/W accessible registers read example
	----------------------------------------
	REG_READ_PROC : process( reg_read_select, reg0, reg1, reg2, reg3 ) is
	begin
		case reg_read_select is
			when "1000" => IP2Bus_Data <= reg0;
			when "0100" => IP2Bus_Data <= reg1;
			when "0010" => IP2Bus_Data <= reg2;
			when "0001" => IP2Bus_Data <= reg3;
			when others => IP2Bus_Data <= (others => '0');
		end case;
	end process REG_READ_PROC;

	----------------------------------------
	-- Clock Dividers
	----------------------------------------
    GEN_1MHZ : process (Bus2IP_Clk) is
    begin
	   if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
	      if Bus2IP_Reset = '1' then
             mhertz_count <= (others => '0');
    	     mhertz_en <= '0';
		  else
    	     mhertz_count <= mhertz_count + 1;
    	     if mhertz_count = "110010" then
    		    mhertz_en <= '1' ;
    		    mhertz_count <= (others => '0');
    	     else 
    		    mhertz_en <= '0';
    	     end if;
          end if;
       end if;
    end process GEN_1MHZ;

    GEN_1KHZ : process (Bus2IP_Clk) is
    begin
       if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
	      if Bus2IP_Reset = '1' then
    	     khertz_count <= (others => '0');
    	     khertz_en <= '0';
          else
    	     if mhertz_en = '1' then
    		    khertz_count <= khertz_count + 1;
    		    if khertz_count = "1111101000" then
    			   khertz_en <= '1';
    			   khertz_count <= (others => '0');
				end if;
    		 else
    			khertz_en <= '0';
    		 end if;
    	  end if;
       end if;
    end process GEN_1KHZ;

	----------------------------------------
	-- User logic display update
	----------------------------------------
	CYC_DISP_PROC : process( Bus2IP_Clk ) is
	begin
		if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
			if Bus2IP_Reset = '1' then
				digit_select <= (others => '0');
				digit_i      <= (others => '1');
				segment_i    <= (others => '1');
			else
  	           if khertz_en = '1' then
		          digit_select <= digit_select + 1 ;
				  case digit_select is
				   	 when "00" => segment_i <= reg0;  digit_i <= "1110";
				   	 when "01" => segment_i <= reg1;  digit_i <= "1101";
					 when "10" => segment_i <= reg2;  digit_i <= "1011";
					 when "11" => segment_i <= reg3;  digit_i <= "0111";
					 when others => null;
				  end case;
			   end if;
			end if;
		end if;
	end process CYC_DISP_PROC;

	----------------------------------------
	-- Simple example to drive IP2Bus signals
	----------------------------------------
	IP2Bus_Ack     <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1) or Bus2IP_WrCE(2) or Bus2IP_WrCE(3) or Bus2IP_RdCE(0) or Bus2IP_RdCE(1) or Bus2IP_RdCE(2) or Bus2IP_RdCE(3);
	IP2Bus_Error   <= '0';
	IP2Bus_Retry   <= '0';
	IP2Bus_ToutSup <= '0';

    digit   <= digit_i;
	segment <= segment_i;

end IMP;
