-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- clut.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        clut.vhd
--
-- Description:     This module creates a 24 bit RGB color from a 4 bits from the color RAM.
--                  The colors are taken from the DOS (Windows) 16 color palette and use
--                  the DOS index.
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              clut.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   rickb  2001-07-17    Verilog Version
--   goran  2002-01-18    VHDL Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library unisim;
use unisim.all;

library opb_color_video_ctrl_v1_00_a;
use opb_color_video_ctrl_v1_00_a.all;

entity CLUT is
  generic (
    C_CHAR_CLUT : integer := 1
    );
  port (
    -- Processor Interface
    Clk        : in  std_logic;
    Reset      : in  std_logic;
    Addr       : in  std_logic_vector(3 downto 0);
    Write      : in  std_logic;
    Write_Data : in  std_logic_vector(23 downto 0);
    Read_Data  : out std_logic_vector(23 downto 0);

    -- Pixel Interface
    Pixel_Clock        : in  std_logic;
    Pixel_Data         : in  std_logic;
    Aligned_Color_Data : in  std_logic_vector(3 downto 0);
    Red_Data           : out std_logic_vector(7 downto 0);
    Green_Data         : out std_logic_vector(7 downto 0);
    Blue_Data          : out std_logic_vector(7 downto 0)
    );

end entity CLUT;

architecture IMP of CLUT is

  component LUT_RAM is
    generic (
      INIT_S : string);
    port (
      -- Processor Interface
      Clk        : in  std_logic;
      Addr       : in  std_logic_vector(3 downto 0);
      Write      : in  std_logic;
      Write_Data : in  std_logic;
      Read_Data  : out std_logic;
      Addr2      : in  std_logic_vector(3 downto 0);
      Read_Data2 : out std_logic);
  end component LUT_RAM;

  type init_array is array (0 to 23) of string(1 to 4);

  constant INIT_V : init_array :=
    ( 0  => "AA00",  1 => "AA00",  2 => "AA00",  3 => "AA00",  4 => "AA00",  5 => "AA00",
      6  => "AA80",  7 => "ABAA",  8 => "CC00",  9 => "CC00", 10 => "CC00", 11 => "CC00" ,
      12 => "CC00", 13 => "CC00", 14 => "CC80", 15 => "CDC0", 16 => "FF00", 17 => "FF00",
      18 => "FF00", 19 => "FF00", 20 => "FF00", 21 => "FF00", 22 => "F080", 23 => "F1F0");

  subtype rgb_data is std_logic_vector(0 to 23);
  type value_array is array (0 to 15) of rgb_data;

  constant RGB_Value : value_array :=
    ( 0  => X"000000",  1 => X"000080",  2 => X"008000",  3 => X"008080",
      4  => X"800000",  5 => X"800080",  6 => X"808000",  7 => X"C0C0C0",
      8  => X"808080",  9 => X"0000FF", 10 => X"00FF00", 11 => X"00FFFF",
      12 => X"FF0000", 13 => X"FF00FF", 14 => X"FFFF00", 15 => X"FFFFFF" );

  signal color_data : std_logic_vector(23 downto 0);

  signal rst : std_logic;
  
begin  -- architecture IMP

  Using_CLUT : if (C_CHAR_CLUT = 1) generate
    CLUT_RAM : for I in 0 to 23 generate
    begin
      LUT_RAM_I : LUT_RAM
        generic map (
          INIT_S => INIT_V(I))          -- [string]
        port map (
          -- Processor Interface
          Clk        => Clk,            -- [in  std_logic]
          Addr       => Addr,           -- [in  std_logic_vector(3 downto 0)]
          Write      => Write,          -- [in  std_logic]
          Write_Data => Write_Data(I),  -- [in  std_logic]
          Read_Data  => Read_Data(I),   -- [out std_logic]
          Addr2      => Aligned_Color_Data,  -- [in  std_logic_vector(3 downto 0)]
          Read_Data2 => color_data(I));      -- [out std_logic]
    end generate CLUT_RAM;
  end generate Using_CLUT;

  No_LUT_RAM: if (C_CHAR_CLUT = 0) generate

    ROM_Table: process (Aligned_Color_Data) is
      variable index : integer;
    begin  -- process ROM_Table
      index := to_integer(unsigned(Aligned_Color_Data));
      color_data <= RGB_Value(index);
    end process ROM_Table;

    Read_Data <= (others => '0');
    
  end generate No_LUT_RAM;
  
  rst <= Reset or not Pixel_Data;

  Out_DFF : process (Pixel_Clock) is
  begin  -- process Out_DFF
    if Pixel_Clock'event and Pixel_Clock = '1' then  -- rising clock edge
      if (rst = '1') then
        Red_Data   <= (others => '0');
        Green_Data <= (others => '0');
        Blue_Data  <= (others => '0');
      else
        Red_Data   <= color_data(23 downto 16);
        Green_Data <= color_data(15 downto 8);
        Blue_Data  <= color_data(7 downto 0);
      end if;
    end if;
  end process Out_DFF;
end architecture IMP;
