-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- opb_color_video_ctrl.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        opb_color_video_ctrl.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              opb_color_video_ctrl.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   goran  2002-01-17    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

entity opb_color_video_ctrl is
  generic (
    C_BASEADDR      : std_logic_vector(0 to 31) := X"FF700000";
    C_HIGHADDR      : std_logic_vector(0 to 31) := X"FF707FFF";
    C_OPB_AWIDTH    : integer                   := 32;
    C_OPB_DWIDTH    : integer                   := 32;
    C_VIDEO_MODE    : integer                   := 6;
    C_BIT_PER_PIXEL : integer                   := 8
    );
  port (
    OPB_Clk : in std_logic;
    OPB_Rst : in std_logic;

    -- OPB signals
    OPB_ABus    : in std_logic_vector(0 to 31);
    OPB_BE      : in std_logic_vector(0 to 3);
    OPB_RNW     : in std_logic;
    OPB_select  : in std_logic;
    OPB_seqAddr : in std_logic;
    OPB_DBus    : in std_logic_vector(0 to 31);

    Char_Video_DBus    : out std_logic_vector(0 to 31);
    Char_Video_errAck  : out std_logic;
    Char_Video_retry   : out std_logic;
    Char_Video_toutSup : out std_logic;
    Char_Video_xferAck : out std_logic;

    -- Video signals
    Pixel_Clock           : in  std_logic;
    VGA_OUT_PIXEL_CLOCK_P : out std_logic;
    VGA_COMP_SYNCH_N      : out std_logic;
    VGA_OUT_BLANK_N       : out std_logic;
    VGA_HSYNCH_N          : out std_logic;
    VGA_VSYNCH_N          : out std_logic;
    VGA_OUT_RED_P         : out std_logic_vector(7 downto 0);
    VGA_OUT_GREEN_P       : out std_logic_vector(7 downto 0);
    VGA_OUT_BLUE_P        : out std_logic_vector(7 downto 0)
    );  
end entity opb_color_video_ctrl;

library Common_v1_00_a;
use Common_v1_00_a.pselect;

library unisim;
use unisim.all;

architecture IMP of opb_color_video_ctrl is

  attribute BOX_TYPE : string;

  component pselect is
    generic (
      C_AB  : integer;
      C_AW  : integer;
      C_BAR : std_logic_vector);
    port (
      A      : in  std_logic_vector(0 to C_AW-1);
      AValid : in  std_logic;
      ps     : out std_logic);
  end component pselect;

  component color_mode_svga_ctrl is
    generic (
      C_VIDEO_MODE    : integer;
      C_BIT_PER_PIXEL : integer := 8
      );
    port (
      -- Processor signals
      OPB_Clk          : in  std_logic;
      OPB_Rst          : in  std_logic;
      Address          : in  std_logic_vector(12 downto 0);
      Write_Data       : in  std_logic_vector(23 downto 0);
      Read_Data        : out std_logic_vector(23 downto 0);
      Char_Mem_Access  : in  std_logic;  -- Access ascii and color information
      Write_Enable     : in  std_logic;  -- write enable
      Color_Map_Access : in  std_logic;  -- Access the color lookup table
      Char_ROM_Access  : in  std_logic;  -- Access the character ROM

      -- Video signals
      pixel_clock           : in  std_logic;  -- pixel clock input
      VGA_OUT_PIXEL_CLOCK_P : out std_logic;  -- pixel clock for the video DAC
      VGA_COMP_SYNCH_N      : out std_logic;  -- composite sync for the video DAC
      VGA_OUT_BLANK_N       : out std_logic;  -- composite blanking for the video DAC
      VGA_HSYNCH_N          : out std_logic;  -- horizontal sync for the VGA output connector
      VGA_VSYNCH_N          : out std_logic;  -- vertical sync for the VGA output connector
      VGA_OUT_RED_P         : out std_logic_vector(7 downto 0);  -- RED DAC data
      VGA_OUT_GREEN_P       : out std_logic_vector(7 downto 0);  -- GREEN DAC data
      VGA_OUT_BLUE_P        : out std_logic_vector(7 downto 0));  -- BLUE DAC data
  end component color_mode_svga_ctrl;

  function Addr_Bits (x, y : std_logic_vector(0 to C_OPB_AWIDTH-1)) return integer is
    variable addr_nor : std_logic_vector(0 to C_OPB_AWIDTH-1);
  begin
    addr_nor := x xor y;
    for i in 0 to C_OPB_AWIDTH-1 loop
      if addr_nor(i) = '1' then return i;
      end if;
    end loop;
    return(C_OPB_AWIDTH);
  end function Addr_Bits;

  constant C_AB : integer := Addr_Bits(C_HIGHADDR, C_BASEADDR);

  component FDRE is
    port (Q  : out std_logic;
          C  : in  std_logic;
          CE : in  std_logic;
          D  : in  std_logic;
          R  : in  std_logic);
  end component FDRE;

  component FDR is
    port (Q : out std_logic;
          C : in  std_logic;
          D : in  std_logic;
          R : in  std_logic);
  end component FDR;

  signal char_video_CS     : std_logic;
  signal char_video_CS_1   : std_logic;
  signal char_video_CS_2   : std_logic;
  signal char_video_CS_2_i : std_logic;

  signal opb_rnw_i : std_logic;

  signal null_16 : std_logic_vector(15 downto 0);
  signal null_2  : std_logic_vector(1 downto 0);
  signal null_1  : std_logic_vector(0 downto 0);

  signal xfer_Ack : std_logic;

  signal read_data : std_logic_vector(23 downto 0);  -- [in]

  signal OPB_ABus_i : std_logic_vector(16 to 17);

  signal write_enable     : std_logic;
  signal Char_Mem_Access  : std_logic;  -- Access ascii and color information
  signal Color_Map_Access : std_logic;  -- Access the color lookup table
  signal Char_ROM_Access  : std_logic;  -- Access the character ROM
  
begin  -- architecture IMP


  -----------------------------------------------------------------------------
  -- Handling the OPB bus interface
  -----------------------------------------------------------------------------

  -- Do the OPB address decoding
  pselect_I : pselect
    generic map (
      C_AB  => C_AB,                    -- [integer]
      C_AW  => C_OPB_AWIDTH,            -- [integer]
      C_BAR => C_BASEADDR)              -- [std_logic_vector]
    port map (
      A      => OPB_ABus,               -- [in  std_logic_vector(0 to C_AW-1)]
      AValid => OPB_select,             -- [in  std_logic]
      ps     => char_video_CS);         -- [out std_logic]


  Char_Video_errAck  <= '0';
  Char_Video_retry   <= '0';
  Char_Video_toutSup <= '0';

  -- Sampling OPB_RNW
  OPB_RNW_DFF : process (OPB_Clk, OPB_Rst)
  begin  -- process OPB_RNW_DFF
    if OPB_Rst = '1' then               -- asynchronous reset (active high)
      opb_rnw_i <= '0';
    elsif OPB_Clk'event and OPB_Clk = '1' then  -- rising clock edge
      opb_rnw_i <= OPB_RNW;
    end if;
  end process OPB_RNW_DFF;

  write_enable <= xfer_Ack and not opb_rnw_i;

  char_video_CS_1_DFF : FDR
    port map (
      Q => char_video_CS_1,             -- [out std_logic]
      C => OPB_Clk,                     -- [in  std_logic]
      D => char_video_CS,               -- [in  std_logic]
      R => xfer_Ack);                   -- [in std_logic]

  char_video_CS_2_DFF : FDR
    port map (
      Q => char_video_CS_2,             -- [out std_logic]
      C => OPB_Clk,                     -- [in  std_logic]
      D => char_video_CS_2_i,           -- [in  std_logic]
      R => xfer_Ack);                   -- [in std_logic]

  char_video_CS_2_i <= not char_video_CS_2 and char_video_CS_1;

  -- Xfer_Ack after 1 clock cycle for writes and after 2 clock cycles for read
  xfer_Ack <= char_video_CS_1 when (opb_rnw_i = '0') else char_video_CS_2;

  Char_Video_xferAck <= xfer_Ack;

  Char_Video_DBus(0 to 7) <= (others => '0');

  OPB_Read_Data_DFFs : for I in 8 to 31 generate
    FDRE_I : FDRE
      port map (
        Q  => Char_Video_DBus(I),       -- [out std_logic]
        C  => OPB_Clk,                  -- [in  std_logic]
        CE => char_video_CS_2_i,        -- [in  std_logic]
        D  => read_data(31-I),          -- [in  std_logic]
        R  => xfer_Ack);                -- [in std_logic]
  end generate OPB_Read_Data_DFFs;

  OPB_ABus_DFF : process (OPB_Clk, OPB_Rst) is
  begin  -- process OPB_ABus_DFF
    if OPB_Rst = '1' then               -- asynchronous reset (active high)
      OPB_ABus_i <= (others => '0');
    elsif OPB_Clk'event and OPB_Clk = '1' then  -- rising clock edge
      OPB_ABus_i <= OPB_ABus(OPB_ABus_i'range);
    end if;
  end process OPB_ABus_DFF;

  char_mem_access  <= '1' when char_video_CS_1 = '1' and OPB_ABus_i(16) = '0'        else '0';
  color_map_access <= '1' when char_video_CS_1 = '1' and OPB_ABus_i(16 to 17) = "10" else '0';
  char_rom_access  <= '1' when char_video_CS_1 = '1' and OPB_ABus_i(16 to 17) = "11" else '0';

  ---------------------------------------------------------------------------
  -- Instanciate the video controller black box
  ---------------------------------------------------------------------------
  color_mode_svga_ctrl_I : color_mode_svga_ctrl
    generic map (
      C_VIDEO_MODE    => C_VIDEO_MODE,  -- [integer]
      C_BIT_PER_PIXEL => C_BIT_PER_PIXEL    -- [integer]
      )
    port map (
      -- Processor signals
      OPB_Clk          => OPB_Clk,      -- [in  std_logic]
      OPB_Rst          => OPB_Rst,      -- [in  std_logic]
      Address          => OPB_ABus(17 to 29),  -- [in  std_logic_vector(12 downto 0)]
      Write_Data       => OPB_DBus(8 to 31),  -- [in  std_logic_vector(23 downto 0)]
      Read_Data        => Read_Data(23 downto 0),  -- [out std_logic_vector(23 downto 0)]
      Char_Mem_Access  => Char_Mem_Access,  -- [in  std_logic] Access ascii and color information
      Write_Enable     => Write_Enable,     -- [in  std_logic] write enable
      Color_Map_Access => Color_Map_Access,  -- [in  std_logic] Access the color lookup table
      Char_ROM_Access  => Char_ROM_Access,  -- [in  std_logic] Access the character ROM

      -- Video signals
      pixel_clock           => pixel_clock,  -- [in  std_logic] pixel clock input
      VGA_OUT_PIXEL_CLOCK_P => VGA_OUT_PIXEL_CLOCK_P,  -- [out std_logic] pixel clock for the video DAC
      VGA_COMP_SYNCH_N      => VGA_COMP_SYNCH_N,  -- [out std_logic] composite sync for the video DAC
      VGA_OUT_BLANK_N       => VGA_OUT_BLANK_N,  -- [out std_logic] composite blanking for the video DAC
      VGA_HSYNCH_N          => VGA_HSYNCH_N,  -- [out std_logic] horizontal sync for the VGA output connector
      VGA_VSYNCH_N          => VGA_VSYNCH_N,  -- [out std_logic] vertical sync for the VGA output connector
      VGA_OUT_RED_P         => VGA_OUT_RED_P,  -- [out std_logic_vector(7 downto 0)] RED DAC data
      VGA_OUT_GREEN_P       => VGA_OUT_GREEN_P,  -- [out std_logic_vector(7 downto 0)] GREEN DAC data
      VGA_OUT_BLUE_P        => VGA_OUT_BLUE_P);  -- [out std_logic_vector(7 downto 0)] BLUE DAC data

end architecture IMP;
